/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.base;

import org.apache.ws.addressing.EndpointReference;
import org.apache.ws.notification.base.impl.MessageTypeNotSupportedException;
import org.apache.ws.notification.base.impl.NoCurrentMessageOnTopicException;
import org.apache.ws.notification.base.impl.SubscribeCreationFailedException;
import org.apache.ws.notification.base.impl.TopicNotSupportedException;
import org.apache.ws.notification.topics.TopicSet;
import org.apache.ws.notification.topics.expression.InvalidTopicExpressionException;
import org.apache.ws.notification.topics.expression.TopicExpression;
import org.apache.ws.notification.topics.expression.TopicPathDialectUnknownException;
import org.apache.ws.resource.PropertiesResource;
import org.apache.ws.resource.properties.query.QueryExpression;
import java.util.Calendar;

/**
 * A resource that implements the WS-BaseNotification NotificationProducer portType.
 * Provides access to the resource's {@link TopicSet} and
 * {@link org.apache.ws.resource.properties.ResourcePropertySet}.
 */
public interface NotificationProducerResource
   extends PropertiesResource
{
   /**
    * Returns the current message for the specified topic (i.e. the last message that was published to the topic).
    *
    * @param topicExpr a topic expression describing EXACTLY ONE topic
    * @return
    */
   Object getCurrentMessage( org.apache.ws.notification.topics.expression.TopicExpression topicExpr )
   throws InvalidTopicExpressionException, 
          TopicNotSupportedException, 
          NoCurrentMessageOnTopicException;

   /**
    * Returns the set of topics supported by this NotificationProducer.
    *
    * @return the set of topics supported by this NotificationProducer
    */
   TopicSet getTopicSet(  );

   /**
    * Publishes the given message to the topic identified by the given topic expression.
    *
    * @param topic a topic expression describing EXACTLY ONE topic
    * @param msg   the notification message to be published - may be a {@link org.w3c.dom.Node}
    *              or an {@link org.apache.xmlbeans.XmlObject}
    */
   void publish( TopicExpression topic,
                 Object          msg )
   throws TopicNotSupportedException, 
          InvalidTopicExpressionException, 
          MessageTypeNotSupportedException;

   /**
    * Subscribe to the topic(s) indicated by the specified topic expression.
    * Notifications for the subscription will be sent to the specified
    * consumer EPR. Other optional parameters may also be specified.
    *
    * @param consumerEPR
    * @param topicExpr
    * @param useNotify
    * @param precondition
    * @param selector
    * @param subscriptionPolicy
    * @param initialTerminationtime
    *
    * @return
    */
   EndpointReference subscribe( EndpointReference consumerEPR,
                                TopicExpression   topicExpr,
                                Boolean           useNotify,
                                QueryExpression   precondition,
                                QueryExpression   selector,
                                Object            subscriptionPolicy,
                                Calendar          initialTerminationtime )
   throws SubscribeCreationFailedException, 
          InvalidTopicExpressionException, 
          TopicPathDialectUnknownException;
}