/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.base;

import org.apache.ws.addressing.EndpointReference;
import org.apache.ws.notification.topics.expression.TopicExpression;
import org.apache.ws.pubsub.NotificationConsumer;
import org.apache.ws.pubsub.NotificationProducer;
import org.apache.ws.resource.PropertiesResource;
import org.apache.ws.resource.lifetime.ScheduledResourceTerminationResource;
import org.apache.ws.resource.properties.query.QueryExpression;

/**
 * A WSN subscription resources. Exposes a subscription's properties, as well as
 * resource lifetime-related state.
 */
public interface Subscription
   extends PropertiesResource,
           org.apache.ws.pubsub.Subscription,
           ScheduledResourceTerminationResource
{
   /**
    * Sets the consumer EPR associated with this subscription.
    *
    * @param consumerRef the consumer EPR to be associated with this subscription
    */
   void setConsumerReference( EndpointReference consumerRef );

   /**
    * Gets the consumer EPR associated with this subscription.
    *
    * @return the consumer EPR associated with this subscription
    */
   EndpointReference getConsumerReference(  );

   /**
    * This method exists for the Pubsub abstraction layer.
    *
    * @param notificationConsumer
    */
   void setNotificationConsumer( NotificationConsumer notificationConsumer );

   /**
    * This method exists for the Pubsub abstraction layer.
    *
    * @param notificationProducer
    */
   void setNotificationProducer( NotificationProducer notificationProducer );

   /**
    * Returns true if this subscription is currently paused, or false otherwise.
    *
    * @return true if this subscription is currently paused, or false otherwise
    */
   boolean isPaused(  );

   /**
    * Sets the policy for the subscription
    *
    * @param policy
    */
   void setPolicy( Object policy );

   /**
    * Gets the policy for this subscription.
    *
    * @return the policy for the subscription, or null if no policy is defined
    */
   Object getPolicy(  );

   /**
    * Sets the precondition for the subscription.
    *
    * @param precondition
    */
   void setPrecondition( QueryExpression precondition );

   /**
    * Gets the precondition associated with this subscription.
    *
    * @return the precondition, or null if no precondition was specified in the subscribe request
    */
   QueryExpression getPrecondition(  );

   /**
    * Gets the producer EPR associated with this subscription. Note, the producer EPR property is read-only.
    *
    * @return the producer EPR associated with this subscription
    */
   EndpointReference getProducerReference(  );

   /**
    * Gets the producer resource associated with this subscription.
    *
    * @return the producer resource
    */
   NotificationProducerResource getProducerResource(  );

   /**
    * Sets the selector for the subscription
    *
    * @param selector
    */
   void setSelector( QueryExpression selector );

   /**
    * Gets the selector associated with this subscription.
    *
    * @return the selector, or null if no selector was specified in the subscribe request
    */
   QueryExpression getSelector(  );

   /**
    * Sets the topic expression associated with this subscription.
    *
    * @param topicExpr the topic expression to be associated with this subscription
    */
   void setTopicExpression( TopicExpression topicExpr );

   /**
    * Gets the topic expression associated with this subscription.
    *
    * @return the topic expression associated with this subscription
    */
   TopicExpression getTopicExpression(  );

   /**
    * Sets the boolean useNotify flag on the subscription to determine if the notification is wrapped in a Notify
    * element.
    *
    * @param useNotify
    */
   void setUseNotify( boolean useNotify );

   /**
    * Wrap notification messages in the notify element?
    *
    * @return true (default) if notify should be used, or false if not
    */
   boolean getUseNotify(  );

   /**
    * Temporarily suspends the delivery of notification messages for this subscription.
    *
    * @throws PauseFailedException if unable to suspend message delivery
    */
   void pause(  )
   throws PauseFailedException;

   /**
    * Resumes the delivery of notification messages for this subscription.
    *
    * @throws ResumeFailedException if unable to resume message delivery
    */
   void resume(  )
   throws ResumeFailedException;
}